type nuhost = (string * string * string)

let name = ref "irc.test"
let version = "0.1"
let start_time = Unix.gettimeofday ()

let dbg msg a =
  prerr_endline ("[" ^ msg ^ "]");
  a

let is_channel str =
  if str == "" then
    false
  else
    match str.[0] with
      | '#' | '+' | '&' -> true
      | _ -> false

let string_map f s =
  let l = String.length s in
  if l = 0 then
    s
  else
    let r = String.create l in
      for i = 0 to l - 1 do
	String.unsafe_set r i (f (String.unsafe_get s i))
      done;
      r

let lowercase_char c =
  if (c >= 'A' && c <= '^') then
    Char.unsafe_chr(Char.code c + 32)
  else
    c

let uppercase_char c =
  if (c >= 'a' && c <= '~') then
    Char.unsafe_chr(Char.code c - 32)
  else
    c

let uppercase s = string_map uppercase_char s
let lowercase s = string_map lowercase_char s

let truncate s len =
  let slen = String.length s in
    if len >= slen then
      s
    else
      Str.string_before s (min slen len)

let nuhost_re = Str.regexp "\\(.*\\)!\\(.*\\)@\\(.*\\)"
let nuhost_of_string str =
  if Str.string_match nuhost_re str 0 then
    (Str.matched_group 1 str,
     Str.matched_group 2 str,
     Str.matched_group 3 str)
  else
    raise Not_found
  
let string_of_nuhost (nick, user, host) =
  nick ^ "!" ^ user ^ "@" ^ host
let nick (nick, user, host) = nick
let user (nick, user, host) = user
let host (nick, user, host) = host
